/*!
**    Notice:  Gubby Copyright 2003 Christian Stigen Larsen
**    Author:  <csl@sublevel3.org>
**
**    $Id: folders.cpp,v 1.7 2003/03/21 16:50:23 cslsublevel3org Exp $
**
**    This file is part of Gubby.
**
**    Gubby is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    Gubby is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Gubby; if not, write to the Free Software
**    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

#include "folders.h"

#include <algorithm>
#include <fstream>
#include <iostream>
#include <iomanip>

Folders::Folders() : total(0) {
	addtrigger("From ");
	addtrigger(" Subject: ");
	addtrigger("  Folder: ");
}

std::string Folders::chop_(std::string line, const std::string& needle) const {
	line.erase(0, needle.size());

	int n(0);

	for ( const char* p=line.c_str();
		p[n]!=' ' && p[n]!='\t' && p[n]!='\0';
		++n);

	line.erase(n, line.size()-n);
	return line;
}

//! Called by grep for each matched line
void Folders::found(const std::string& line, const int triggerno) {

	std::string chopped( chop_( line, gettrigger(triggerno)));
	mail m;
	Folder cur;

	switch ( triggerno ) {

	case 0: // From
		from_ = chopped;
		break;

	case 1: // Subject
		subj_ = chopmin_(line, gettrigger(triggerno));
		break;

	case 2: // Folder
		m.subj = subj_;
		m.from = from_;

		subj_ = "";
		from_ = "";

		cur.setname(chopped);
		cur.push(m);

		// Insert folder or insert mail in folder
		for ( Folders::iterator p(begin()); p != end(); ++p )
			if ( (*p) == cur ) {
				(*p).push(m);
				newest_ = (*p).name();
				return;
			}

		newest_ = chopped;
		push_back(cur);

	default:
		break;
	}
}

unsigned long Folders::totalmail_() const {
	unsigned long ret(0);

	for ( Folders::const_iterator f = begin(); f != end(); ++f )
		ret += (*f).vmail.size();

	return ret;
}

void Folders::read(const std::string& file) {
	clear();
	
	std::ifstream f(file.c_str());
	find(f);

	sort(begin(), end());
	reverse(begin(), end());
	total = totalmail_();
}
